from dataclasses import dataclass
from typing import Union

from jwt_helper._signmethods import SignMethod
from jwt_helper._load_key_from_disk import load_key_from_disk

import base64

SECRET_METHODS = {
    "plain": lambda val: val,
    "base64plain": lambda val: base64.b64decode(val.encode("utf-8")).decode("utf8"),
    "base64key": base64.b64decode,
    "keyfile": load_key_from_disk,
}


@dataclass
class Issuer:
    """This class represents an Issuer

    An Issuer is an auth-server which creates JWT Tokens. We list supported Methods for validating there cookies and the Secret belonging to it here.
    We have a list of supported methods here so Attackers cant swap the signing method around (for example switching from HS to RS Signature and
    using the Public Key as HMAC secret)

    Attributes:
        name (str): The name of the issuer which is also included in the JWT Header
        secret (bytes | string): either the symetric HMAC secret or the Public key used to verify the Signature
        methods (list[SignMethod]): The list of supported Signin Methods for the specified Issuer

    """

    name: str
    secret: Union[bytes, str]
    methods: list[SignMethod]

    def __repr__(self):
        return f"Issuer(name: {self.name}, methods: {self.methods})"

    @staticmethod
    def from_dict(data: dict):
        # translate sign methods
        methods = []
        for method in data["methods"]:
            methods.append(SignMethod[method])
        # load key the specified way
        key_type = data["secret"]["type"]
        secret_value = data["secret"]["value"]

        if not (
            key_type == "keyfile"
            or key_type == "base64plain"
            or key_type == "base64key"
            or key_type == "plain"
        ):
            raise ValueError(f"keytype {key_type} not supported")
        secret = SECRET_METHODS[key_type](secret_value)

        return Issuer(data["name"], secret, methods)



import jwt
import datetime

from dataclasses import dataclass
from typing import Union
from cryptography.hazmat.primitives.asymmetric.types import PrivateKeyTypes

from ._signmethods import SignMethod


@dataclass
class JWTEncoder:
    """This class helps encoding JWTs

    Attributes:
        issuer (str): The Issuer who created the JWT.
        signmethod (SignMethod): The Algorithm used to create the JWT.
        secret (bytes | PrivateKeyTypes | str): The secret used to create the JWT signature
    """

    issuer: str
    signmethod: SignMethod
    secret: Union[bytes, PrivateKeyTypes, str]

    def __repr__(self):
        return f"JWTEncoder(issuer={self.issuer!r}, signmethod={self.signmethod!r})"

    def create_jwt(self, headers: dict, body: dict, valid_minutes: int) -> str:
        """This Function will create the JWT Token

        Args:
            headers (dict): additional data for the JWT Header.
            body (dict): the JWT Payload.
            valid_minutes (int): the lifetime of the token.

        Returns:
            str: The JWT Token generated
        """

        valid_until = datetime.datetime.now() + datetime.timedelta(
            minutes=valid_minutes
        )

        jwt_headers = {
            "iss": self.issuer,
            "iat": datetime.datetime.now().timestamp(),
            "nbf": datetime.datetime.now().timestamp(),
            "exp": valid_until.timestamp(),
        }

        for key, value in headers.items():
            if key not in ["iss", "iat", "nbf", "exp"]:
                jwt_headers[key] = value

        token = jwt.encode(
            body,
            self.secret,
            headers=jwt_headers,
            algorithm=self.signmethod.value,
        )
        return token



from setuptools import setup
with open('jwt_helper/_version.py') as infile:
    version_line = infile.read()

version_line = version_line.replace("__version__","")
version_line = version_line.replace("=","")
version_line = version_line.replace(" ","")
version_line = version_line.replace(" ","")
version = version_line.replace("\"","")
setup(

    version=version
)



from pathlib import Path
from typing import Optional, Union

from cryptography.hazmat.primitives.asymmetric.types import PrivateKeyTypes
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.backends import default_backend


def load_key_from_disk(
    path: str, secret: Optional[bytes] = None
) -> Union[bytes, PrivateKeyTypes]:
    """Load Key from Disk with a given Path

    This Function loads the specified key from disks and returns it as Bytes or Private Key type (if there is a secret specified).

    Args:
        path (str): path to the keyfile
        secret (str, optional): secret for the keyfile. Defaults to None.

    Returns:
        Union[bytes, PrivateKeyTypes]: the loaded key
    """

    file = Path(path)
    key_bytes = file.read_bytes()

    # if no secret is specified return the bytes as is
    if secret is None:
        return key_bytes

    return serialization.load_pem_private_key(
        key_bytes, password=secret, backend=default_backend()
    )
